/*
 * Netris -- A free networked version of Tetris
 * Copyright (C) 1994  Mark Weaver <Mark_Weaver@brown.edu>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: curses.c,v 1.27 1994/12/06 02:03:09 mhw Exp $
 */

#include "netris.h"
#include <sys/types.h>
#include <unistd.h>
#include <curses.h>
#include <string.h>
#include <stdlib.h>

static void PlotBlock1(int scr, int y, int x, BlockType type);
static MyEventType KeyGenFunc(EventGenRec *gen, MyEvent *event);

static EventGenRec keyGen =
		{ NULL, 0, FT_read, STDIN_FILENO, KeyGenFunc, EM_key };

static int boardYPos[MAX_SCREENS], boardXPos[MAX_SCREENS];
static int statusYPos, statusXPos;

ExtFunc void InitScreens(void)
{
	MySigSet oldMask;

	BlockSignals(&oldMask, SIGINT, 0);
	initscr();
	AtExit(CleanupScreens);
	RestoreSignals(NULL, &oldMask);
	cbreak();
	noecho();
	AddEventGen(&keyGen);
	move(0, 0);
	addstr("Netris ");
	addstr(version_string);
	addstr(" (C) 1994  Mark Weaver         \"netris -h\" for more info");
	statusYPos = 22;
	statusXPos = 0;
}

ExtFunc void CleanupScreens(void)
{
	RemoveEventGen(&keyGen);
	endwin();
}

ExtFunc void InitScreen(int scr)
{
	int y, x;

	if (scr == 0)
		boardXPos[scr] = 1;
	else
		boardXPos[scr] = boardXPos[scr - 1] +
					2 * boardWidth[scr - 1] + 3;
	boardYPos[scr] = 22;
	if (statusXPos < boardXPos[scr] + 2 * boardWidth[scr] + 3)
		statusXPos = boardXPos[scr] + 2 * boardWidth[scr] + 3;
	for (y = boardVisible[scr] - 1; y >= 0; --y) {
		move(boardYPos[scr] - y, boardXPos[scr] - 1);
		addch('|');
		move(boardYPos[scr] - y, boardXPos[scr] + 2 * boardWidth[scr]);
		addch('|');
	}
	for (y = boardVisible[scr]; y >= -1; y -= boardVisible[scr] + 1) {
		move(boardYPos[scr] - y, boardXPos[scr] - 1);
		addch('+');
		for (x = boardWidth[scr] - 1; x >= 0; --x)
			addstr("--");
		addch('+');
	}
}

ExtFunc void CleanupScreen(int scr)
{
}

static void PlotBlock1(int scr, int y, int x, BlockType type)
{
	move(boardYPos[scr] - y, boardXPos[scr] + 2 * x);
	switch (type) {
		case BT_none:
			addstr("  ");
			break;
		case -BT_piece1:
			if (standoutEnable)
				standout();
			addstr("$$");
			if (standoutEnable)
				standend();
			break;
		case BT_piece1:
			if (standoutEnable)
				standout();
			addstr("[]");
			if (standoutEnable)
				standend();
			break;
		default:
			assert(0);
	}
}

ExtFunc void PlotBlock(int scr, int y, int x, BlockType type)
{
	if (y >= 0 && y < boardVisible[scr] && x >= 0 && x < boardWidth[scr])
		PlotBlock1(scr, y, x, type);
}

ExtFunc void PlotUnderline(int scr, int x, int flag)
{
	move(boardYPos[scr] + 1, boardXPos[scr] + 2 * x);
	addstr(flag ? "==" : "--");
}

ExtFunc void ShowDisplayInfo(void)
{
	move(statusYPos - 8, statusXPos);
	printw("Seed = %d", initSeed);
	if (robotEnable) {
		move(statusYPos - 6, statusXPos);
		if (fairRobot)
			addstr("Controlled by a fair robot");
		else
			addstr("Controlled by a robot");
	}
	if (opponentFlags & SCF_usingRobot) {
		move(statusYPos - 5, statusXPos);
		if (opponentFlags & SCF_fairRobot)
			addstr("The opponent is a fair robot");
		else
			addstr("The opponent is a robot");
	}
}

ExtFunc void UpdateOpponentDisplay(void)
{
	static char buf[128];

	sprintf(buf, "Playing %s@%s", opponentName, opponentHost);
	buf[78] = 0;
	move(1, 0);
	addstr(buf);
	clrtoeol();
}

ExtFunc void ShowPause(int pausedByMe, int pausedByThem)
{
	move(statusYPos - 3, statusXPos);
	if (pausedByThem)
		addstr("Game paused by opponent");
	else
		clrtoeol();
	move(statusYPos - 2, statusXPos);
	if (pausedByMe)
		addstr("Game paused by you");
	else
		clrtoeol();
}

ExtFunc void Message(char *s)
{
	static int line = 0;

	move(statusYPos - 20 + line, statusXPos);
	addstr(s);	/* XXX Should truncate long lines */
	clrtoeol();
	line = (line + 1) % 10;
	move(statusYPos - 20 + line, statusXPos);
	clrtoeol();
}

ExtFunc void RefreshScreen(void)
{
	static char timeStr[2][32];
	time_t theTime;

	time(&theTime);
	strftime(timeStr[0], 30, "%I:%M %p", localtime(&theTime));
	/* Just in case the local curses library sucks */
	if (strcmp(timeStr[0], timeStr[1]))
	{
		move(statusYPos, statusXPos);
		addstr(timeStr[0]);
		strcpy(timeStr[1], timeStr[0]);
	}
	move(boardYPos[0] + 1, boardXPos[0] + 2 * boardWidth[0] + 1);
	refresh();
}

static MyEventType KeyGenFunc(EventGenRec *gen, MyEvent *event)
{
	if (MyRead(gen->fd, &event->u.key, 1))
		return E_key;
	else
		return E_none;
}

